SystemOrganization addCategory: #'Seaside-Adaptors-Hyper'!

WAServerAdaptor subclass: #WAHyperAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Hyper'!

!WAHyperAdaptor commentStamp: 'pmm 1/1/2009 20:06' prior: 0!
A WAHyperAdaptor makes Seaside run on Hyper.

Instance Variables
	server:		<HTTPServer>

server
	- The Hyper HTTP server
!

!WAHyperAdaptor class methodsFor: 'testing' stamp: 'pmm 4/17/2010 12:11'!
isAbstract
	^ false! !

!WAHyperAdaptor methodsFor: 'private' stamp: 'pmm 1/1/2009 20:04'!
basicStart
	server portNumber: self port.
	server start! !

!WAHyperAdaptor methodsFor: 'private' stamp: 'pmm 1/1/2009 20:05'!
basicStop
	server stop! !

!WAHyperAdaptor methodsFor: 'private' stamp: 'pmm 1/1/2009 22:38'!
convertCookiesFrom: aWAResonse to: aNativeResponse
	aWAResonse do: [ :each |
		aNativeResponse cookie: each oldNetscapeString ]! !

!WAHyperAdaptor methodsFor: 'initialization' stamp: 'pmm 1/1/2009 22:40'!
initialize
	super initialize.
	server := HTTPServer new.
	server multiThreading: true.
	"Use MessageSend instead of blocks because Squeak is gayer than AIDS"
	server responseBlock: (MessageSend
		receiver: self 
		selector: #handleRequest:).
	server internalErrorHandler: (MessageSend
		receiver: self
		selector: #internalErrorResponseFor:)! !

!WAHyperAdaptor methodsFor: 'testing' stamp: 'pmm 1/1/2009 20:10'!
isRunning
	^server isStarted! !

!WAHyperAdaptor methodsFor: 'testing' stamp: 'pmm 1/1/2009 20:12'!
isStarting
	^false! !

!WAHyperAdaptor methodsFor: 'testing' stamp: 'pmm 1/1/2009 20:11'!
isStopped
	^server isStarted not! !

!WAHyperAdaptor methodsFor: 'testing' stamp: 'pmm 1/1/2009 20:11'!
isStopping
	^server isStopping! !

!WAHyperAdaptor methodsFor: 'private' stamp: 'pmm 4/17/2010 12:00'!
postFieldsOf: aNativeRequest into: aRequestFields
	"break encapsulation - This line is likely unnecessary because #requestBodyFor:
		will have already done this after extracting the body."
	aNativeRequest ensureFullRead. ""
	aNativeRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self codec url decode: value value ]
			ifFalse: [
				| isoCodec |
				isoCodec := GRCodec forEncoding: 'ISO-8859-1'.
				WAFile new
					fileName: (self codec url decode: value filename);
					contentType: value contentType seasideMimeType;
					"does a 1:1 mapping from character values to bytes"
					contents: (isoCodec encode: value value) asByteArray;
					yourself ].
		aRequestFields at: key add: data ]! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:01'!
requestAddressFor: aNativeRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aNativeRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:08'!
requestBodyFor: aNativeRequest
	"This only works for POST and PUT requests... LAME!!"

	aNativeRequest isPost ifTrue: [
		| body |
		"Nasty nastiness but we need to get the raw body before Swazoo parses
		the POST data. Then we need to manually reinject the data to allow it to
		be parsed (since the data has already been pulled off the socket)."
		body := aNativeRequest postData stream nextBytes: aNativeRequest contentLength.
		aNativeRequest parsePostDataFrom: (HyOctetReadStream onByteArray: body).
		aNativeRequest postData setParsed.
		^ body ].
	aNativeRequest isPut
		ifTrue: [ ^aNativeRequest putData ].
	^nil! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:09'!
requestCookiesFor: aNativeRequest
	^ WARequestCookie fromString: aNativeRequest cookie! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:09'!
requestFieldsFor: aNativeRequest
	| fields |
	fields := WARequestFields new.
	aNativeRequest isPost
		ifTrue: [ self postFieldsOf: aNativeRequest into: fields ].
	^ fields! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:10'!
requestHeadersFor: aNativeRequest
	| hyperFields fields |
	hyperFields := aNativeRequest headers fields.
	fields := Dictionary new: hyperFields size.
	hyperFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:10'!
requestMethodFor: aNativeRequest
	^ aNativeRequest methodName! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:10'!
requestUrlFor: aNativeRequest
	| url |
	url := ((WAUrl absolute: aNativeRequest uri printString)
		decodedWith: self codec).
	aNativeRequest isEncrypted
		ifTrue: [ url scheme: 'https' ].
	^ url! !

!WAHyperAdaptor methodsFor: 'converting-request' stamp: 'pmm 4/17/2010 12:10'!
requestVersionFor: aNativeRequest
	| array |
	array := aNativeRequest httpVersion.
	^ WAHttpVersion major: array first minor: array second! !

!WAHyperAdaptor methodsFor: 'converting' stamp: 'pmm 4/17/2010 12:05'!
responseFrom: aRequestContext 
	| hyperResponse |
	hyperResponse := HTTPResponse new.
	aRequestContext response headers keysAndValuesDo: [ :key :value | 
		key = 'Content-Type' 
			ifTrue: [ hyperResponse contentType: value greaseString ]
			ifFalse:  [
				| field fieldClass |
				fieldClass := HeaderField classForFieldName: key.
				field := fieldClass 
					newForFieldName: key
					withValueFrom: value greaseString.
				hyperResponse headers addField: field ] ].
	self 
		convertCookiesFrom: aRequestContext response
		to: hyperResponse.
	hyperResponse code: aRequestContext response status asNumber.
	hyperResponse entity: aRequestContext response contents.
	^ hyperResponse! !

!WAHyperAdaptor methodsFor: 'private' stamp: 'pmm 1/1/2009 20:06'!
shutDown
	HTTPServer shutDown! !
